/***********************************************************************
 * 
 *   Palmcon Consultoria Ltda. Copyright 2002.
 * 
 *   PalmFLD.c
 * 
 *   Biblioteca padro de manipulao de Fields
 * 
 *   Histrico:
 *   =========
 * 
 *   18/05/2002 - JSM
 *      - Criao da biblioteca
 *      - Adicionada funo FldHandleToField
 *      - Adicionada funo FldStrToField
 *      - Adicionada funo FldFieldToStr
 *      - Adicionada funo FldSetEditable
 *      - Adicionada funo FldSetNumeric
 *      - Adicionada funo FldSetSingleLine
 * 
 ***********************************************************************/
#include <PalmOS.h>
#include "PalmFLD.h"
#include "palmstd_lib.h"

/******************************************************************************************
 *  Atributos do Field
 *   typedef struct {
 *   	unsigned int usable				:1;	// Set if part of ui 
 *	    unsigned int visible			:1;	// Set if drawn, used internally
 *	    unsigned int editable			:1;	// Set if editable
 *	    unsigned int singleLine			:1;	// Set if only a single line is displayed
 *   	unsigned int hasFocus       	:1; // Set if the field has the focus
 *   	unsigned int dynamicSize		:1; // Set if height expands as text is entered
 *   	unsigned int insPtVisible		:1;	// Set if the ins pt is scolled into view
 *   	unsigned int dirty				:1;	// Set if user modified
 *   	unsigned int underlined			:2;	// text underlined mode
 *   	unsigned int justification		:2;	// text alignment
 *   	unsigned int autoShift			:1;	// Set if auto case shift
 *   	unsigned int hasScrollBar		:1;	// Set if the field has a scroll bar
 *   	unsigned int numeric			:1;	// Set if numeric, digits and secimal separator only
 *   } FieldAttrType;
 ********************************************************************************************/


/***********************************************************************
 * 
 *   FldHandleToText
 * 
 *   Faz com que o texto de um field seja equivalente ao contedo de 
 * um handle.  uma funo mais baixo nvel, utilizada por outras que  
 * facilitam a manipulao de Fields mais amigavelmente.  
 *   
 *   Pega o handle antigo do Field, libera memria e faz com que o 
 * field utilize o novo handle.  
 *   
 *   Retorno: FieldPtr   - Se o field passado era inexistente, vai ser
 *                        NULL. Seno,  um ponteiro para o field.
 * 
 ****************************************************************************************************************/
extern FieldPtr FldHandleToField(unsigned int fieldID, MemHandle txtH)
{
   MemHandle   oldTxtH;
   FormPtr     frm = FrmGetActiveForm();
   FieldType * fldP;

   // Pega o handle atual utilizado pelo field para manipular o texto
   fldP     = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, fieldID));
   ErrNonFatalDisplayIf(!fldP, "missing field");
   oldTxtH  = FldGetTextHandle(fldP);
   
   // Seta o field para utilizar o novo handle
   FldSetTextHandle(fldP, txtH);
   FldDrawField(fldP);

   // Libera memria do handle antigo
   if (oldTxtH) 
      MemHandleFree(oldTxtH);
   
   return fldP;
} 



/***********************************************************************
 * 
 *   FldStrToField
 * 
 *   Copia uma string para um field, diretamente.
 *     - Aloca um handle do tamanho da string
 *     - Duplica a string no handle
 *     - Faz o field apontar para o handle
 *   
 *   Retorno: FieldPtr   - Se o field passado era inexistente, vai ser
 *                        NULL. Seno,  um ponteiro para o field.
 * 
 ****************************************************************************************************************/
extern FieldPtr FldStrToField(unsigned int fieldID, const char * strP)
{
   MemHandle   txtH;
   FieldPtr    FldPtr;
   char      * txtPtr;
   
   // Aloca o handle para a cpia da string.
   txtH  = MemHandleNew(StrLen(strP) + 1);
   if (!txtH)
      return NULL;

   // Trava o handle e copia a string para o handle e destrava o handle
   txtPtr = MemHandleLock(txtH);
   StrCopy(txtPtr, strP);
   MemHandleUnlock(txtH);
   
   // Faz o field utilizar o handle
   FldPtr = FldHandleToField(fieldID, txtH);
   
   return FldPtr;
} 



/***********************************************************************
 * 
 *   FldFieldToStr
 * 
 *   Copia o contedo de um field para uma string previamente alocada
 *   
 *   PS: No custa nada repetir: J deve haver uma string previamente
 * alocada para que esta funo funcione.   
 *   
 ****************************************************************************************************************/
extern void FldFieldToStr(char * Destino,  unsigned int fieldID)
{
   FieldPtr field;
   char *  OldTextPtr;

   // Descobre qual  o ponteiro para o field e se o field possui texto
   field      = GetObjectPtr(fieldID);
   OldTextPtr = FldGetTextPtr(field);

   // Se o field possui texto, copia para a string. Seno, zera a string
   if (OldTextPtr != NULL)
      StrCopy(Destino, OldTextPtr);
   else
      Destino[0] = '\0';
}



/***********************************************************************
 * 
 *   FldSetEditable
 * 
 *   Permite que um field seja editvel ou no
 *   
 ****************************************************************************************************************/
extern void FldSetEditable(unsigned int fieldId, Boolean estado)
{
   FieldAttrType atributos;
   FldGetAttributes(GetObjectPtr(fieldId), &atributos);
   atributos.editable = estado;
   FldSetAttributes(GetObjectPtr(fieldId), &atributos);   
}



/***********************************************************************
 * 
 *   FldSetNumeric
 * 
 *   Exige que um field seja apenas numrico ou no
 *   
 ****************************************************************************************************************/
extern void FldSetNumeric(unsigned int fieldId, Boolean numerico)
{
   FieldAttrType atributos;
   FldGetAttributes(GetObjectPtr(fieldId), &atributos);
   atributos.numeric = numerico;
   FldSetAttributes(GetObjectPtr(fieldId), &atributos);   
}



/***********************************************************************
 * 
 *   FldSetSingleLine
 * 
 *   Permite que um field possua uma linha (true) ou mais (false)
 *   
 ****************************************************************************************************************/
extern void FldSetSingleLine(unsigned int fieldId, Boolean umalinha)
{
   FieldAttrType atributos;
   FldGetAttributes(GetObjectPtr(fieldId), &atributos);
   atributos.singleLine = umalinha;
   FldSetAttributes(GetObjectPtr(fieldId), &atributos);   
}



/***********************************************************************
 * 
 *   
 *   FUNES ESQUECIDAS PELA HUMANIDADE (E PELA PALMCON)
 *     (podem at serem utilizadas posteriormente...)
 *   
 *   
 *   
 ****************************************************************************************************************
extern void      f_CampoTextoParaString(char * Destino,  unsigned int fieldID);
extern FieldPtr  f_StringParaCampoPtr(FieldPtr FldPtr, char * strP);
extern void      f_CampoPtrParaString(char * Destino,  FieldPtr field);


/***********************************************************************
 * 
 *   f_StringParaCampoPtr - Idem a FldStrToField, recebe PTR invs de ID
 *   
 ****************************************************************************************************************
extern FieldPtr f_StringParaCampoPtr(FieldPtr FldPtr, char * strP)
{
   void *      txtH;
//   FieldPtr    FldPtr;
   
   // get some space in which to stash the string.
   txtH  = MemHandleNew(StrLen(strP) + 1);
   if (!txtH)
      return NULL;

   // copy the string to the locked void *.
   StrCopy(MemHandleLock(txtH), strP);

   // unlock the string void *.
   MemHandleUnlock(txtH);
   
   // set the field to the void *
   FldPtr = SetFieldPtrTextFromHandle(FldPtr, txtH);
   
   // redraw the field
   FldDrawField(FldPtr);
   
   return FldPtr;
} 


/***********************************************************************
 * 
 *   f_CampoPtrParaString - Idem a FldFieldToStr, recebe PTR invs de ID
 *   
 ****************************************************************************************************************
extern void f_CampoPtrParaString(char * Destino,  FieldPtr field)
{
//   FieldPtr field;
   char *  OldTextPtr;

   // Deve-se testar se o *Destino != NULL, pra ver se ele ja nao aponta para
   // algum lugar, e se ele apontar, desaloca-lo.
   
// No Code Here... We Dont know it.

   // Pega a string do field, cria uma nova area de memoria para ela e
   // copia a string para a nova area de memoria, retornando o valor por
   // "Destino".
//   field = GetObjectPtr( fieldID );
   OldTextPtr = FldGetTextPtr ( field );
   if (OldTextPtr != NULL)
      StrCopy(Destino, OldTextPtr);
   else
      Destino[0] = '\0';
}


/***********************************************************************
 * 
 *   SetFieldPtrTextFromHandle - Idem a FldHandleToField, recebe PTR invs de ID
 *   
 ****************************************************************************************************************
extern FieldPtr SetFieldPtrTextFromHandle(FieldPtr fldP, void * txtH)
{
   void *      oldTxtH;
   FormPtr     frm = FrmGetActiveForm();
//   FieldPtr    fldP;


   // get the field and the field's current text void *.
//   fldP     = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, fieldID));
   ErrNonFatalDisplayIf(!fldP, "missing field");
   oldTxtH  = FldGetTextHandle(fldP);
   
   // set the field's text to the new text.
   FldSetTextHandle(fldP, txtH);
   FldDrawField(fldP);

   // free the void * AFTER we call FldSetTextHandle().
   if (oldTxtH) 
      MemHandleFree(oldTxtH);
   
   return fldP;
} 
/************** EOF **************/